"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.printNewEditionOperationHandler = void 0;
const web3_js_1 = require("@solana/web3.js");
const spl_token_1 = require("@solana/spl-token");
const bn_js_1 = __importDefault(require("bn.js"));
const tokenMetadata_1 = require("../../../programs/tokenMetadata");
const transactionBuilders_1 = require("../transactionBuilders");
const errors_1 = require("../../../errors");
exports.printNewEditionOperationHandler = {
    handle: (operation, metaplex) => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b;
        const { originalMint, newMint = web3_js_1.Keypair.generate(), newMintAuthority = metaplex.identity(), newUpdateAuthority = newMintAuthority.publicKey, newOwner = newMintAuthority.publicKey, newFreezeAuthority, payer = metaplex.identity(), tokenProgram, associatedTokenProgram, confirmOptions, } = operation.input;
        // Original NFT.
        const originalMetadata = yield tokenMetadata_1.MetadataAccount.pda(originalMint);
        const originalEdition = yield tokenMetadata_1.OriginalEditionAccount.pda(originalMint);
        const originalEditionAccount = tokenMetadata_1.OriginalEditionAccount.fromMaybe(yield metaplex.rpc().getAccount(originalEdition));
        if (!originalEditionAccount.exists) {
            throw new errors_1.AccountNotFoundError(originalEdition, 'OriginalEdition', `Ensure the provided mint address for the original NFT [${originalMint.toBase58()}] ` +
                `is correct and that it has an associated OriginalEdition PDA.`);
        }
        const edition = new bn_js_1.default(originalEditionAccount.data.supply, 'le').add(new bn_js_1.default(1));
        const originalEditionMarkPda = yield tokenMetadata_1.EditionMarkerAccount.pda(originalMint, edition);
        // New NFT.
        const newMetadata = yield tokenMetadata_1.MetadataAccount.pda(newMint.publicKey);
        const newEdition = yield tokenMetadata_1.PrintEditionAccount.pda(newMint.publicKey);
        const lamports = yield (0, spl_token_1.getMinimumBalanceForRentExemptMint)(metaplex.connection);
        const newAssociatedToken = yield (0, spl_token_1.getAssociatedTokenAddress)(newMint.publicKey, newOwner, false, tokenProgram, associatedTokenProgram);
        const sharedInput = {
            lamports,
            edition,
            newMint,
            newMetadata,
            newEdition,
            newMintAuthority,
            newUpdateAuthority,
            newOwner,
            newAssociatedToken,
            newFreezeAuthority,
            payer,
            originalMetadata,
            originalEdition,
            originalEditionMarkPda,
            tokenProgram,
            associatedTokenProgram,
        };
        let transactionBuilder;
        if (operation.input.via === 'vault') {
            transactionBuilder = (0, transactionBuilders_1.printNewEditionBuilder)(Object.assign({ via: 'vault', vaultAuthority: operation.input.vaultAuthority, safetyDepositStore: operation.input.safetyDepositStore, safetyDepositBox: operation.input.safetyDepositBox, vault: operation.input.vault, tokenVaultProgram: operation.input.tokenVaultProgram }, sharedInput));
        }
        else {
            const originalTokenAccountOwner = (_a = operation.input.originalTokenAccountOwner) !== null && _a !== void 0 ? _a : metaplex.identity();
            const originalTokenAccount = (_b = operation.input.originalTokenAccount) !== null && _b !== void 0 ? _b : (yield (0, spl_token_1.getAssociatedTokenAddress)(originalMint, originalTokenAccountOwner.publicKey, false, tokenProgram, associatedTokenProgram));
            transactionBuilder = (0, transactionBuilders_1.printNewEditionBuilder)(Object.assign({ via: 'token', originalTokenAccountOwner,
                originalTokenAccount }, sharedInput));
        }
        const { signature } = yield metaplex
            .rpc()
            .sendAndConfirmTransaction(transactionBuilder, undefined, confirmOptions);
        return {
            mint: newMint,
            metadata: newMetadata,
            edition: newEdition,
            associatedToken: newAssociatedToken,
            transactionId: signature,
        };
    }),
};
//# sourceMappingURL=printNewEditionOperationHandler.js.map