"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.printNewEditionBuilder = void 0;
const token_1 = require("../../../programs/token");
const tokenMetadata_1 = require("../../../programs/tokenMetadata");
const shared_1 = require("../../../shared");
const printNewEditionBuilder = (params) => {
    const { 
    // Data.
    lamports, edition, 
    // New NFT.
    newMint, newMetadata, newEdition, newMintAuthority, newUpdateAuthority, newOwner, newAssociatedToken, newFreezeAuthority, payer, 
    // Master NFT.
    originalMetadata, originalEdition, originalEditionMarkPda, 
    // Programs.
    tokenProgram, associatedTokenProgram, 
    // Instruction keys.
    createAccountInstructionKey, initializeMintInstructionKey, createAssociatedTokenInstructionKey, mintToInstructionKey, printNewEditionInstructionKey = 'printNewEdition', } = params;
    let printNewEditionViaBuilder;
    if (params.via === 'vault') {
        printNewEditionViaBuilder = (0, tokenMetadata_1.mintNewEditionFromMasterEditionViaVaultProxyBuilder)({
            edition,
            newMetadata,
            newEdition,
            masterEdition: originalEdition,
            newMint,
            editionMarkPda: originalEditionMarkPda,
            newMintAuthority,
            payer,
            vaultAuthority: params.vaultAuthority,
            safetyDepositStore: params.safetyDepositStore,
            safetyDepositBox: params.safetyDepositBox,
            vault: params.vault,
            newMetadataUpdateAuthority: newUpdateAuthority,
            metadata: originalMetadata,
            tokenVaultProgram: params.tokenVaultProgram,
            instructionKey: printNewEditionInstructionKey,
        });
    }
    else {
        printNewEditionViaBuilder = (0, tokenMetadata_1.mintNewEditionFromMasterEditionViaTokenBuilder)({
            edition,
            newMetadata,
            newEdition,
            masterEdition: originalEdition,
            newMint,
            editionMarkPda: originalEditionMarkPda,
            newMintAuthority,
            payer,
            tokenAccountOwner: params.originalTokenAccountOwner,
            tokenAccount: params.originalTokenAccount,
            newMetadataUpdateAuthority: newUpdateAuthority,
            metadata: originalMetadata,
            instructionKey: printNewEditionInstructionKey,
        });
    }
    return (shared_1.TransactionBuilder.make()
        // Create the mint account and send one token to the holder.
        .add((0, token_1.createMintAndMintToAssociatedTokenBuilder)({
        lamports,
        decimals: 0,
        amount: 1,
        createAssociatedToken: true,
        mint: newMint,
        payer,
        mintAuthority: newMintAuthority,
        owner: newOwner,
        associatedToken: newAssociatedToken,
        freezeAuthority: newFreezeAuthority,
        tokenProgram,
        associatedTokenProgram,
        createAccountInstructionKey,
        initializeMintInstructionKey,
        createAssociatedTokenInstructionKey,
        mintToInstructionKey,
    }))
        // Mint new edition.
        .add(printNewEditionViaBuilder));
};
exports.printNewEditionBuilder = printNewEditionBuilder;
//# sourceMappingURL=printNewEditionBuilder.js.map