"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OriginalEditionAccount = void 0;
const buffer_1 = require("buffer");
const mpl_token_metadata_1 = require("@metaplex-foundation/mpl-token-metadata");
const tokenMetadata_1 = require("./..");
const shared_1 = require("../../../shared");
const errors_1 = require("../../../errors");
class OriginalEditionAccount extends shared_1.BaseAccount {
    static pda(mint) {
        return __awaiter(this, void 0, void 0, function* () {
            return shared_1.Pda.find(tokenMetadata_1.TokenMetadataProgram.publicKey, [
                buffer_1.Buffer.from('metadata', 'utf8'),
                tokenMetadata_1.TokenMetadataProgram.publicKey.toBuffer(),
                mint.toBuffer(),
                buffer_1.Buffer.from('edition', 'utf8'),
            ]);
        });
    }
    static recognizes(buffer) {
        return this.recognizesV1(buffer) || this.recognizesV2(buffer);
    }
    static recognizesV1(buffer) {
        return (buffer === null || buffer === void 0 ? void 0 : buffer[0]) === mpl_token_metadata_1.Key.MasterEditionV1;
    }
    static recognizesV2(buffer) {
        return (buffer === null || buffer === void 0 ? void 0 : buffer[0]) === mpl_token_metadata_1.Key.MasterEditionV2;
    }
    static from(unparsedAccount) {
        if (!this.recognizes(unparsedAccount.data)) {
            throw new errors_1.UnexpectedAccountError(unparsedAccount.publicKey, 'MasterEditionV1|MasterEditionV2');
        }
        const accountData = this.recognizesV1(unparsedAccount.data) ? mpl_token_metadata_1.MasterEditionV1 : mpl_token_metadata_1.MasterEditionV2;
        return new OriginalEditionAccount(this.parse(unparsedAccount, accountData));
    }
    static fromMaybe(maybe) {
        return maybe.exists ? this.from(maybe) : maybe;
    }
}
exports.OriginalEditionAccount = OriginalEditionAccount;
//# sourceMappingURL=OriginalEditionAccount.js.map