import { createMintAndMintToAssociatedTokenBuilder } from "../../../programs/token";
import { mintNewEditionFromMasterEditionViaTokenBuilder, mintNewEditionFromMasterEditionViaVaultProxyBuilder, } from "../../../programs/tokenMetadata";
import { TransactionBuilder } from "../../../shared";
export const printNewEditionBuilder = (params) => {
    const { 
    // Data.
    lamports, edition, 
    // New NFT.
    newMint, newMetadata, newEdition, newMintAuthority, newUpdateAuthority, newOwner, newAssociatedToken, newFreezeAuthority, payer, 
    // Master NFT.
    originalMetadata, originalEdition, originalEditionMarkPda, 
    // Programs.
    tokenProgram, associatedTokenProgram, 
    // Instruction keys.
    createAccountInstructionKey, initializeMintInstructionKey, createAssociatedTokenInstructionKey, mintToInstructionKey, printNewEditionInstructionKey = 'printNewEdition', } = params;
    let printNewEditionViaBuilder;
    if (params.via === 'vault') {
        printNewEditionViaBuilder = mintNewEditionFromMasterEditionViaVaultProxyBuilder({
            edition,
            newMetadata,
            newEdition,
            masterEdition: originalEdition,
            newMint,
            editionMarkPda: originalEditionMarkPda,
            newMintAuthority,
            payer,
            vaultAuthority: params.vaultAuthority,
            safetyDepositStore: params.safetyDepositStore,
            safetyDepositBox: params.safetyDepositBox,
            vault: params.vault,
            newMetadataUpdateAuthority: newUpdateAuthority,
            metadata: originalMetadata,
            tokenVaultProgram: params.tokenVaultProgram,
            instructionKey: printNewEditionInstructionKey,
        });
    }
    else {
        printNewEditionViaBuilder = mintNewEditionFromMasterEditionViaTokenBuilder({
            edition,
            newMetadata,
            newEdition,
            masterEdition: originalEdition,
            newMint,
            editionMarkPda: originalEditionMarkPda,
            newMintAuthority,
            payer,
            tokenAccountOwner: params.originalTokenAccountOwner,
            tokenAccount: params.originalTokenAccount,
            newMetadataUpdateAuthority: newUpdateAuthority,
            metadata: originalMetadata,
            instructionKey: printNewEditionInstructionKey,
        });
    }
    return (TransactionBuilder.make()
        // Create the mint account and send one token to the holder.
        .add(createMintAndMintToAssociatedTokenBuilder({
        lamports,
        decimals: 0,
        amount: 1,
        createAssociatedToken: true,
        mint: newMint,
        payer,
        mintAuthority: newMintAuthority,
        owner: newOwner,
        associatedToken: newAssociatedToken,
        freezeAuthority: newFreezeAuthority,
        tokenProgram,
        associatedTokenProgram,
        createAccountInstructionKey,
        initializeMintInstructionKey,
        createAssociatedTokenInstructionKey,
        mintToInstructionKey,
    }))
        // Mint new edition.
        .add(printNewEditionViaBuilder));
};
//# sourceMappingURL=printNewEditionBuilder.js.map