import { Buffer } from 'buffer';
import { MasterEditionV1, MasterEditionV2, Key } from '@metaplex-foundation/mpl-token-metadata';
import { TokenMetadataProgram } from "./..";
import { BaseAccount, Pda } from "../../../shared";
import { UnexpectedAccountError } from "../../../errors";
export class OriginalEditionAccount extends BaseAccount {
    static async pda(mint) {
        return Pda.find(TokenMetadataProgram.publicKey, [
            Buffer.from('metadata', 'utf8'),
            TokenMetadataProgram.publicKey.toBuffer(),
            mint.toBuffer(),
            Buffer.from('edition', 'utf8'),
        ]);
    }
    static recognizes(buffer) {
        return this.recognizesV1(buffer) || this.recognizesV2(buffer);
    }
    static recognizesV1(buffer) {
        return (buffer === null || buffer === void 0 ? void 0 : buffer[0]) === Key.MasterEditionV1;
    }
    static recognizesV2(buffer) {
        return (buffer === null || buffer === void 0 ? void 0 : buffer[0]) === Key.MasterEditionV2;
    }
    static from(unparsedAccount) {
        if (!this.recognizes(unparsedAccount.data)) {
            throw new UnexpectedAccountError(unparsedAccount.publicKey, 'MasterEditionV1|MasterEditionV2');
        }
        const accountData = this.recognizesV1(unparsedAccount.data) ? MasterEditionV1 : MasterEditionV2;
        return new OriginalEditionAccount(this.parse(unparsedAccount, accountData));
    }
    static fromMaybe(maybe) {
        return maybe.exists ? this.from(maybe) : maybe;
    }
}
//# sourceMappingURL=OriginalEditionAccount.js.map