import { OriginalEditionAccount, PrintEditionAccount } from "./..";
import { BaseAccount } from "../../../shared";
import { UnexpectedAccountError } from "../../../errors";
export class OriginalOrPrintEditionAccount extends BaseAccount {
    static async pda(mint) {
        return OriginalEditionAccount.pda(mint);
    }
    static recognizes(buffer) {
        return OriginalEditionAccount.recognizes(buffer) || PrintEditionAccount.recognizes(buffer);
    }
    static from(unparsedAccount) {
        if (!this.recognizes(unparsedAccount.data)) {
            throw new UnexpectedAccountError(unparsedAccount.publicKey, 'MasterEditionV1|MasterEditionV2|EditionV1');
        }
        const account = OriginalEditionAccount.recognizes(unparsedAccount.data)
            ? OriginalEditionAccount.from(unparsedAccount)
            : PrintEditionAccount.from(unparsedAccount);
        return new OriginalOrPrintEditionAccount(account);
    }
    static fromMaybe(maybe) {
        return maybe.exists ? this.from(maybe) : maybe;
    }
    isOriginal() {
        return 'maxSupply' in this.data;
    }
    isPrint() {
        return !this.isOriginal();
    }
}
//# sourceMappingURL=OriginalOrPrintEditionAccount.js.map