/// <reference types="node" />
import { AccountInfo, PublicKey } from '@solana/web3.js';
import { Buffer } from 'buffer';
declare type AccountDataConstructor<T> = {
    name: string;
    fromAccountInfo(accountInfo: AccountInfo<Buffer>, offset?: number): [T, number];
};
export declare type Account<T> = AccountInfo<T> & {
    publicKey: PublicKey;
};
export declare type MaybeAccount<T> = (Account<T> & {
    exists: true;
}) | {
    publicKey: PublicKey;
    exists: false;
};
export declare type UnparsedAccount = Account<Buffer>;
export declare type UnparsedMaybeAccount = MaybeAccount<Buffer>;
export declare abstract class BaseAccount<T> implements Account<T> {
    readonly publicKey: PublicKey;
    readonly executable: boolean;
    readonly owner: PublicKey;
    readonly lamports: number;
    readonly data: T;
    readonly rentEpoch?: number | undefined;
    readonly exists: true;
    protected constructor(accountValue: Account<T>);
    static parse<T>(unparsedAccount: UnparsedAccount, accountData: AccountDataConstructor<T>): Account<T>;
    static parseMaybe<T>(unparsedMaybeAccount: UnparsedMaybeAccount, accountData: AccountDataConstructor<T>): MaybeAccount<T>;
}
export {};
